@@ -30,13 +30,12 @@ Once the operation's class has been defined, make an entry in the `name_to_class
 dictonary in this module. After doing this, the operation should work throughout the
 hera-py toolkit (interpreter, debugger, assembler, etc.)!
 
-Author:  Ian Fisher (iafisher@protonmail.com)
+Author:  Ian Fisher (iafisher@fastmail.com)
 Version: July 2019
 """
 import json
 import sys
 from contextlib import suppress
-from typing import Dict, List, Optional, Union
 
 from hera import stdlib
 from hera.data import Constant, DataLabel, HERAError, Label, Location, Messages, Token
@@ -68,7 +67,7 @@ class AbstractOperation:
         else:
             self.loc = None
 
-    def typecheck(self, symbol_table: Dict[str, int]) -> Messages:
+    def typecheck(self, symbol_table: "Dict[str, int]") -> Messages:
         """
         Type-check the operation. Subclasses do not generally need to override this
         method, as long as they provide a P class field listing their parameter types.
@@ -83,14 +82,14 @@ class AbstractOperation:
 
         return messages.extend(check_arglist(self.P, self.tokens, symbol_table))
 
-    def convert(self) -> List["AbstractOperation"]:
+    def convert(self) -> "List[AbstractOperation]":
         """
         Convert the pseudo-operation into a list of real operations. Only pseudo-ops
         need to override this method.
         """
         return [self]
 
-    def assemble(self) -> bytes:
+    def assemble(self) -> "Optional[bytes]":
         """
         Assemble the operation into a 16-bit string. Subclasses do not generally need
         to override this method, as long as they provide a BITV class field.
@@ -150,10 +149,10 @@ REGISTER = "REGISTER"
 REGISTER_OR_LABEL = "REGISTER_OR_LABEL"
 STRING = "STRING"
 LABEL_TYPE = "LABEL_TYPE"
-I16 = range(-2 ** 15, 2 ** 16)
+I16 = range(-(2 ** 15), 2 ** 16)
 I16_OR_LABEL = "I16_OR_LABEL"
 U16 = range(2 ** 16)
-I8 = range(-2 ** 7, 2 ** 8)
+I8 = range(-(2 ** 7), 2 ** 8)
 I8_OR_LABEL = "I8_OR_LABEL"
 U5 = range(2 ** 5)
 U4 = range(2 ** 4)
@@ -276,7 +275,8 @@ class RelativeBranch(Branch):
 
 
 class DebuggingOperation(AbstractOperation):
-    pass
+    def assemble(self):
+        return None
 
 
 class DataOperation(AbstractOperation):
@@ -422,7 +422,7 @@ class SUB(BinaryOp):
         # uints, left - right - borrow might not be.
         result = to_u16((left - right - borrow) & 0xFFFF)
 
-        vm.flag_carry = left >= right
+        vm.flag_carry = left >= (right + borrow)
         vm.flag_overflow = from_u16(result) != from_u16(left) - from_u16(right) - borrow
 
         return result
@@ -573,7 +573,7 @@ class DEC(AbstractOperation):
 
         vm.set_zero_and_sign(result)
         vm.flag_overflow = from_u16(result) != from_u16(original) - value
-        vm.flag_carry = original < value
+        vm.flag_carry = original >= value
         vm.pc += 1
 
     def assemble(self):
@@ -671,7 +671,7 @@ class ASL(UnaryOp):
     ASL(Rd, Rb)
       Shift the value of Rb one bit to the left, and store the result in Rd. If the
       carry-block is off and the carry is set, then the bit 1 is shifted in on the
-      left; otherwise 0 is shifted in. The bit shifted out becomes the carry flag.
+      right; otherwise 0 is shifted in. The bit shifted out becomes the carry flag.
 
       ASL sets the sign flag when the result is negative, and the zero flag when the
       result is zero.
@@ -695,30 +695,24 @@ class ASL(UnaryOp):
 
 class ASR(UnaryOp):
     """
-    ASL(Rd, Rb)
-      Shift the value of Rb one bit to the left, and store the result in Rd. If the
-      carry-block is off and the carry is set, then the bit 1 is shifted in on the
-      left; otherwise 0 is shifted in. The bit shifted out becomes the carry flag.
+    ASR(Rd, Rb)
+      Shift the value of Rb one bit to the right, and store the result in Rd. The bit
+      shifted out becomes the carry flag.
 
-      ASL sets the sign flag when the result is negative, and the zero flag when the
+      ASR sets the sign flag when the result is negative, and the zero flag when the
       result is zero.
 
-      The only difference between ASL and LSL is that ASL will additionally set the
-      overflow flag to the same value it would receive after executing ADD(Rd, Rb, Rb).
+      Unlike LSL, LSR and ASL, ASR ignores the carry flag regardless of whether the
+      carry-block is set and always shifts in 0 on the left. The result of ASR is always
+      one-half of the original signed value, rounded towards negative infinity.
     """
 
     BITV = "0011 AAAA 0101 BBBB"
 
     @staticmethod
     def calculate(vm, arg):
-        # This is a little messy because right shift in Python rounds towards
-        # negative infinity (7 >> 1 == -4) but in HERA it rounds towards zero
-        # (7 >> 1 == -3).
         if arg & 0x8000:
-            if arg & 0x0001:
-                result = ((arg >> 1) | 0x8000) + 1
-            else:
-                result = arg >> 1 | 0x8000
+            result = arg >> 1 | 0x8000
         else:
             result = arg >> 1
 
@@ -1860,7 +1854,7 @@ def disassemble(v: int) -> AbstractOperation:
     raise HERAError("bit pattern does not correspond to HERA instruction")
 
 
-def match_bitvector(pattern: str, v: int) -> Union[List, bool]:
+def match_bitvector(pattern: str, v: int) -> "Union[List, bool]":
     """
     Try to match the 16-bit integer `v` against `pattern`. Return a list of extracted
     arguments if `v` matches, or False otherwise.
@@ -1907,7 +1901,7 @@ def match_bitvector(pattern: str, v: int) -> Union[List, bool]:
     return args
 
 
-def substitute_bitvector(pattern: str, args: List[int]) -> bytes:
+def substitute_bitvector(pattern: str, args: "List[int]") -> bytes:
     """
     Given a 16-bit pattern and a list of arguments, substitute the arguments into the
     pattern to yield a two-byte machine operation.
@@ -1929,7 +1923,7 @@ def substitute_bitvector(pattern: str, args: List[int]) -> bytes:
     return bytes([high_byte, low_byte])
 
 
-def substitute_half_a_bitvector(pattern: str, args: List[int]) -> int:
+def substitute_half_a_bitvector(pattern: str, args: "List[int]") -> int:
     """
     Helper function for `substitute_bitvector`. Takes an 8-bit pattern and returns a
     single byte.
@@ -1967,10 +1961,12 @@ def check_arglist(argtypes, args, symbol_table):
             err = check_string(got)
         elif expected == I16_OR_LABEL:
             err = check_in_range(
-                got, symbol_table, lo=-2 ** 15, hi=2 ** 16, labels=True
+                got, symbol_table, lo=-(2 ** 15), hi=2 ** 16, labels=True
             )
         elif expected == I8_OR_LABEL:
-            err = check_in_range(got, symbol_table, lo=-2 ** 7, hi=2 ** 8, labels=True)
+            err = check_in_range(
+                got, symbol_table, lo=-(2 ** 7), hi=2 ** 8, labels=True
+            )
         elif isinstance(expected, range):
             err = check_in_range(got, symbol_table, lo=expected.start, hi=expected.stop)
         else:
@@ -1981,7 +1977,7 @@ def check_arglist(argtypes, args, symbol_table):
     return messages
 
 
-def check_register(arg) -> Optional[str]:
+def check_register(arg) -> "Optional[str]":
     """
     Check that `arg` is a register. Return an error message as a string if it is not.
     """
@@ -1994,7 +1990,7 @@ def check_register(arg) -> Optional[str]:
             return "expected register"
 
 
-def check_register_or_label(arg, symbol_table: Dict[str, int]) -> Optional[str]:
+def check_register_or_label(arg, symbol_table: "Dict[str, int]") -> "Optional[str]":
     """
     Check that `arg` is a register or a label. Return an error message as a string if it
     is not.
@@ -2017,7 +2013,7 @@ def check_register_or_label(arg, symbol_table: Dict[str, int]) -> Optional[str]:
         return "expected register or label"
 
 
-def check_label(arg) -> Optional[str]:
+def check_label(arg) -> "Optional[str]":
     """Check that `arg` is a label. Return an error message as a string if it is not."""
     if arg.type == Token.SYMBOL:
         return None
@@ -2025,7 +2021,7 @@ def check_label(arg) -> Optional[str]:
         return "expected label"
 
 
-def check_string(arg) -> Optional[str]:
+def check_string(arg) -> "Optional[str]":
     """
     Check that `arg` is a string literal. Return an error message as a string if it is
     not.
@@ -2036,7 +2032,7 @@ def check_string(arg) -> Optional[str]:
         return None
 
 
-def check_in_range(arg, symbol_table, *, lo, hi, labels=False) -> Optional[str]:
+def check_in_range(arg, symbol_table, *, lo, hi, labels=False) -> "Optional[str]":
     """
     Check that `arg` is an integer (or a symbol resolving to an integer) within the
     bounds established by `lo` and `hi`.
